<?php

namespace Ems\Controllers;

use Exception\Messages;
use Service\LogService;
use Service\OAuth\OAuthCustomImpl;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;

use Ems\Services\MinhaHistoriaService;
use Ems\Util\Utils;

class MinhaHistoriaController {
    protected $minhaHistoriaService;

    public function __construct(MinhaHistoriaService $minhaHistoriaService) {
        $this->minhaHistoriaService = $minhaHistoriaService;
    }

    public function solicitaAlteracao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            if ($row["NRTIPOSOLICITACAO"] == 16) {
                $nrSolicitacao = $this->minhaHistoriaService->solicitaAlteracaoComunica($row);
            } else {
                $nrSolicitacao = $this->minhaHistoriaService->solicitaAlteracao($row);
            }

            $dataset = [
                'success' => true,
                'nrsolicitacao' => $nrSolicitacao,
            ];

            $response->addDataSet(new DataSet('solicitaAlteracao', $dataset));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getSolicitacaoFile(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();

            $nrSolicitacao = (int) $row['NRSOLICITACAO'];

            $file = $this->minhaHistoriaService->
                getSolicitacaoFile($nrSolicitacao);

            $response->addDataSet(new DataSet('getSolicitacaoFile', $file));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

}
