<?php

namespace Ems\Logic\ProcessamentoSolicitacao;

use Ems\Util\Utils;
use Util\{ DateUtil, NovoCodigo, Repositories };

class ProcessamentoSolicitacaoRemanejamento extends ProcessamentoSolicitacao {
    
    public function efetivar() {
        $itens = $this->getItensAssociativeArray();
        
        $nrorg          = Utils::getParamters()->getNrorg();
        $nrvinculom     = $this->getSolicitacao()->getNrvinculom();
        $dtcompetencia  = DateUtil::getDataDeString(Utils::getParamters()->getDtmescomptencDiaCorrente(),DateUtil::FORMATO_BRASILEIRO,true);
        
        $paramOrg = Utils::getEntityManager()->getRepository(Repositories::GPE_PARAMORG)->findOneBy(['nrorg' => $nrorg]);
        $nrtipoestrutlegal = !empty($paramOrg) ? $paramOrg->getNrtpestrutlegal() : null;

        // Transferência/Remanejamento
        $nrtipotransfer = $itens['NRTIPOTRANSFER']  ?? null;
        $nrtpmovtransfm = $itens['NRTPMOVTRANSFM']  ?? null;
        $nrmotivotransf = $itens['NRMOTIVOTRANSF']  ?? null;
        $nrestruturam   = $itens['NRESTRUTURAM']    ?? null;
        $dtinimoviment  = isset($itens['DTINIMOVIMENT']) ? DateUtil::getDataDeString(substr($itens['DTINIMOVIMENT'],0,10), DateUtil::FORMATO_BRASILEIRO, true) : null;
        $dtfimmoviment  = isset($itens['DTFIMMOVIMENT']) ? DateUtil::getDataDeString(substr($itens['DTFIMMOVIMENT'],0,10), DateUtil::FORMATO_BRASILEIRO, true) : null;
    
        $estrutura_nova = Utils::getEntityManager()->getRepository(Repositories::ESTRUTURAM)->findOneBy(array("nrorg"=> $nrorg,"nrestruturam" => $nrestruturam));
        $result = Utils::getEntityManager()->getRepository(Repositories::GPE_MOVIMENTACAO)->findBy(array("nrorg"=> $nrorg,"nrvinculom" => $nrvinculom,"dtinimoviment" => $dtinimoviment));

        // SALÁRIO
        $dtaltesalario  = isset($itens['DTALTESALARIO']) ? DateUtil::getDataDeString(substr($itens['DTALTESALARIO'],0,10), 'd/m/Y', true) : null;
        $nrtipoaltera   = $itens['NRTIPOALTERA']        ?? null;
        $idtpsalario    = $itens['IDTPSALARIO']         ?? null;
        $nrocorrencia   = $itens['NROCORRENCIA']        ?? null;
        $nrtpmodalidsal = $itens['NRTPMODALIDSAL']  ?? null; 
        $novoVlrSalario = $itens['VRSALARIO']           ?? $itens['NOVOVALORSALARIO'] ?? null;

        // ESCALA
        $nrturno        = $itens['NRTURNO']             ??  null;
        $nrmotivoescala = $itens['NRMOTIVOESCALA']      ??  null;
        $nrescalatrabm  = $itens['NRESCALATRABM']       ??  null;
        $dtiniescala    = isset($itens['DTINIESCALA'])  ?   DateUtil::getDataDeString(substr($itens['DTINIESCALA'],0,10), DateUtil::FORMATO_BRASILEIRO, true) : null;
        $dtAtual        = DateUtil::getDataAtual()->format(DateUtil::FORMATO_BRASILEIRO);


        if($result){
            $estrutura_historico = Utils::getEntityManager()->getRepository(Repositories::ESTRUTURAM)->findBy(array("nrorg"=> $nrorg,"nrestruturam" => $result->getNrestruturam()));
            if($estrutura_historico->getNrtipoestrutura() == $estrutura_nova->getNrtipoestrutura())
                throw new \Exception("Já existe uma movimentação para este tipo de estrutura nesta data");
        }
        if($nrtipotransfer == 1){
            if($estrutura_nova->getNrtipoestrutura() != $nrtipoestrutlegal && $estrutura_nova->getNrtipoestrutura() != 10)
                throw new \Exception("A nova estrutura legal deve ser do mesmo tipo que a antiga.");
        }
        else if($estrutura_nova->getNrtipoestrutura() == $nrtipoestrutlegal)
                throw new \Exception("A nova estrutura legal deve ser de tipo diferente que a antiga.");
   
        if($dtAtual !== $itens['DTINIMOVIMENT'] && !DateUtil::comparaDatas($dtinimoviment, DateUtil::getDataDeString($dtAtual), DateUtil::DEPOIS)){
                throw new \Exception("A movimentação não pode ser retroativa");
        }


        $gpeMovimentacao = Repositories::GPE_MOVIMENTACAO;
        $movimentacao = new $gpeMovimentacao;
        $movimentacao->setNrorg($nrorg);
        $movimentacao->setNrvinculom($nrvinculom);
        $movimentacao->setNrtipoestrutura($estrutura_nova->getNrtipoestrutura());
        $movimentacao->setNrestruturam($nrestruturam);
        $movimentacao->setNrmotivotransf($nrmotivotransf);
        $movimentacao->setNrtipotransfer($nrtipotransfer);
        $movimentacao->setNrtpmovtransfm($nrtpmovtransfm);
        $movimentacao->setDtfimmoviment($dtfimmoviment);
        $movimentacao->setDtinimoviment($dtinimoviment);
        $movimentacao->setDtinclusao(DateUtil::getDataAtual(false));
        $movimentacao->setNrorginclusao(Utils::getParamters()->getNrorg());
        $movimentacao->setCdoperinclusao(Utils::getParamters()->getCdoperador());
        $movimentacao->setNrmovimentacao(NovoCodigo::geraCodigo(Utils::getParamters()->getNrorg(), 'GPE_MOVIMENTACAO', 1, 12, true));
        Utils::getEntityManager()->persist($movimentacao);
        Utils::getEntityManager()->flush();
        
        $vinc = ProcessamentoSolicitacaoUtil::retornaUltimoHistoricoVinculoCompetencia($nrorg, $nrvinculom, $dtcompetencia);
        if($vinc->getDtmescompetenc() == $dtcompetencia){
            $vinc->setDtultatu(DateUtil::getDataAtual());
            $vinc->setNrorgultatu(Utils::getParamters()->getNrorg());
            $vinc->setCdoperultatu(Utils::getParamters()->getCdoperador());
            if($nrtipotransfer == 1)
                $vinc->setNrestrutlegal($nrestruturam);
            else 
                $vinc->setNrestrutgeren($nrestruturam);
            Utils::getEntityManager()->persist($vinc);
            Utils::getEntityManager()->flush();
        }
        else
        ProcessamentoSolicitacaoUtil::novoHistoricoVinculo($nrorg, $nrvinculom, $dtcompetencia, $vinc->getNrsitufuncm(), $vinc->getNrcargo(), $vinc->getNrescalatrabm(),
            $vinc->getIdcontribuisind(), $vinc->getNrdependir(), $vinc->getNrdependsfam(), $vinc->getDtbaseferias(), $vinc->getIdremuneracao(),
            $vinc->getIdtppagamento(), (($nrtipotransfer == 1)?$nrestruturam:$vinc->getNrestrutlegal()),(($nrtipotransfer == 1)?$vinc->getNrestrutgeren():$nrestruturam) , $vinc->getNrtpmodalidsal(), $vinc->getIdmultvinc(),
            $vinc->getNrfuncao(), $vinc->getCdcontribindividual(), $vinc->getNrtpmovtransfm(), $vinc->getDtfimcontrdetermin(),
            $vinc->getNrestrutsind(), $vinc->getConfidencial()
        );


        if($nrescalatrabm && $nrturno && $dtiniescala) {
            $result = Utils::getEntityManager()->getRepository(Repositories::GPE_ALTEESCALA)->findBy(array("nrorg"=> $nrorg,"nrvinculom" => $nrvinculom,"dtiniescala" => $dtiniescala));

            if($result) {
                throw new \Exception("Já existe uma alteração de escala para este vinculo nesta data");
            }
            
            ProcessamentoSolicitacaoUtil::atualizaAlteEscala(
                $nrorg,
                $nrvinculom,
                $nrescalatrabm,
                DateUtil::subtraiIntervalo($dtiniescala,1,DateUtil::DIA)->format(DateUtil::FORMATO_BRASILEIRO),
                DateUtil::getDataAtual(true)->format(DateUtil::FORMATO_BRASILEIRO),
                Utils::getParamters()->getCdoperador()
            );

            ProcessamentoSolicitacaoUtil::atualizaEscalaVinculoh(
                $nrorg,
                $nrvinculom,
                $nrescalatrabm,
                DateUtil::getDataAtual(true)->format(DateUtil::FORMATO_BRASILEIRO),
                Utils::getParamters()->getCdoperador()
            );
            
            $alteEscala = ProcessamentoSolicitacaoUtil::novoAlteEscala($nrorg, $nrvinculom, $nrturno, $nrmotivoescala, $nrescalatrabm, $dtiniescala);
        }


        if($nrtipoaltera && $novoVlrSalario && $idtpsalario) {
            $alteSalario = ProcessamentoSolicitacaoUtil::novoAlteSalario($nrorg, $nrvinculom, $dtaltesalario, $nrocorrencia, $nrtipoaltera, $nrtpmodalidsal, $idtpsalario, $novoVlrSalario);
        }
        
    }
    
    public function rejeitar() {}
}