<?php
namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;
use Util\NovoCodigo;
use Ems\Util\Utilities;

class ControleEntregaEPIRepository{
    protected $entityManager;
    protected $connection;

    public function __construct(
        EntityManager $entityManager
    ){
        $this->connection      = $entityManager->getConnection();
        $this->entityManager   = $entityManager;
    }

    public function getControleEntregaEPI(array $params): array {
        $sql = <<<SQL
            SELECT
                SST_HISTORICOEPI.NRHISTORICOEPI,
                SST_HISTORICOEPI.NRORG,
                SST_HISTORICOEPI.NRVINCULOM,
                SST_HISTORICOEPI.NREPI,
                SST_HISTORICOEPI.QTFORNECIDAEPI,
                SST_HISTORICOEPI.NRPARCNEGOCIO,
                SST_HISTORICOEPI.NRCTRLAPROVEPI,
                SST_HISTORICOEPI.DSMODELOEPI,
                SST_HISTORICOEPI.IDPROTECAOEFICAZ,
                SST_HISTORICOEPI.IDCTRLENTREGA,
                TO_CHAR(SST_HISTORICOEPI.DTENTREGAEPI, 'DD/MM/YYYY') AS DTENTREGAEPI,
                TO_CHAR(SST_HISTORICOEPI.DTEFETIVA, 'DD/MM/YYYY') AS DTEFETIVA,
                SST_HISTORICOEPI.NRMOTIVODEVEPI,
                TO_CHAR(SST_HISTORICOEPI.DTDEVOLUCAO, 'DD/MM/YYYY') AS DTDEVOLUCAO,
                CASE
                    WHEN SST_HISTORICOEPI.IDPROTECAOEFICAZ = 'S'
                        THEN 'Sim'
                    ELSE 'Não'
                END DSPROTECAOEFICAZ,
                VN.NMVINCULOM,
                EPI.NMEPI,
                PCN.NMPRINCIPALPARC,
                MDP.NMMOTIVODEVEPI
            FROM
                SST_HISTORICOEPI
                INNER JOIN GPE_VINCULOM       VN ON VN.NRORG = SST_HISTORICOEPI.NRORG
                                              AND VN.NRVINCULOM = SST_HISTORICOEPI.NRVINCULOM
                LEFT JOIN GPE_VINCULOH        VH ON VH.NRORG = SST_HISTORICOEPI.NRORG
                                              AND VH.NRVINCULOM = VN.NRVINCULOM
                                              AND VH.DTMESCOMPETENC = (SELECT 
                                                                         MAX(DTMESCOMPETENC) 
                                                                       FROM 
                                                                         GPE_VINCULOH 
                                                                       WHERE 
                                                                         NRORG = VH.NRORG 
                                                                         AND NRVINCULOM = VH.NRVINCULOM 
                                                                         AND DTMESCOMPETENC <= :P_DTMESCOMPETENC)
                INNER JOIN SST_EPI            EPI ON EPI.NREPI = SST_HISTORICOEPI.NREPI
                                          AND EPI.NRORG = SST_HISTORICOEPI.NRORG
                LEFT JOIN PARCNEGOCIO        PCN ON PCN.NRORG = SST_HISTORICOEPI.NRORG
                                             AND PCN.NRPARCNEGOCIO = SST_HISTORICOEPI.NRPARCNEGOCIO
                LEFT JOIN SST_MOTIVODEVEPI   MDP ON (MDP.NRORG = SST_HISTORICOEPI.NRORG OR MDP.NRORG = :P_NRORG_PADRAO)
                                                  AND MDP.NRMOTIVODEVEPI = SST_HISTORICOEPI.NRMOTIVODEVEPI
            WHERE
                (SST_HISTORICOEPI.NRORG = :P_NRORG OR SST_HISTORICOEPI.NRORG = :P_NRORG_PADRAO)
                AND (:P_NRESTRUTURAM = -1 OR VH.NRESTRUTGEREN = :P_NRESTRUTURAM OR VH.NRESTRUTGEREN IN (
                    SELECT
                        HIER.NRESTRUTURAMREL
                    FROM HIERARQUIAORG HIER
                    WHERE HIER.NRORG = :P_NRORG
                    START WITH
                        HIER.NRESTRUTURAMSUP = :P_NRESTRUTURAM
                    CONNECT BY NOCYCLE
                        HIER.NRESTRUTURAMSUP = PRIOR HIER.NRESTRUTURAMREL
                        AND PRIOR HIER.NRORG = :P_NRORG
                ))
            ORDER BY VN.NMVINCULOM ASC
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $controleEntregaEPI = $std->fetchAll();

        return $controleEntregaEPI;
    }

    public function getEpi(array $params): array {
        $sql = <<<SQL
            SELECT
                EPI.*,
                MEPI.NMTIPOMEDIDAEPI,
                TEPI.NMTIPOEPI
            FROM
                SST_EPI EPI
            JOIN SST_TIPOMEDIDAEPI MEPI ON
                EPI.NRTIPOMEDIDAEPI = MEPI.NRTIPOMEDIDAEPI AND
                ( EPI.NRORG = MEPI.NRORG OR MEPI.NRORG = :P_NRORG_PADRAO )
            JOIN SST_TIPOEPI TEPI ON
                EPI.NRTIPOEPI = TEPI.NRTIPOEPI AND
                ( EPI.NRORG = TEPI.NRORG OR TEPI.NRORG = :P_NRORG_PADRAO )
            WHERE EPI.NRORG = :P_NRORG
            ORDER BY EPI.NMEPI
SQL;
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $epi = $std->fetchAll();

        return $epi;
    }

    public function newHistoricoEpi($row) {
        try {
            $nrorg = $row['NRORG'];
            $nrssthistoricoepi = NovoCodigo::geraCodigo($nrorg, 'SST_HISTORICOEPI', 1, 22, true);

            $sql = <<<SQL
                    INSERT INTO SST_HISTORICOEPI (
                        NRHISTORICOEPI,
                        NRMOTIVODEVEPI,
                        NRORG,
                        NRVINCULOM,
                        NREPI,
                        NRPARCNEGOCIO,
                        DTENTREGAEPI,
                        IDCTRLENTREGA,
                        DTEFETIVA,
                        QTFORNECIDAEPI,
                        DTDEVOLUCAO,
                        DSMODELOEPI,
                        NRCTRLAPROVEPI,
                        IDPROTECAOEFICAZ,
                        IDATIVO,
                        NRORGINCLUSAO,
                        DTINCLUSAO,
                        CDOPERINCLUSAO,
                        NRORGULTATU,
                        DTULTATU,
                        CDOPERULTATU
                    ) VALUES (
                        :P_NRHISTORICOEPI,
                        :P_NRMOTIVODEVEPI,
                        :P_NRORG,
                        :P_NRVINCULOM,
                        :P_NREPI,
                        :P_NRPARCNEGOCIO,
                        TO_DATE(:P_DTENTREGAEPI,'DD/MM/YYYY'),
                        :P_IDCTRLENTREGA,
                        TO_DATE(:P_DTEFETIVA,'DD/MM/YYYY'),
                        :P_QTFORNECIDAEPI,
                        :P_DTDEVOLUCAO,
                        :P_DSMODELOEPI,
                        :P_NRCTRLAPROVEPI,
                        :P_IDPROTECAOEFICAZ,
                        :P_IDATIVO,
                        :P_NRORGINCLUSAO,
                        TO_DATE(:P_DTINCLUSAO,'DD/MM/YYYY'),
                        :P_CDOPERINCLUSAO,
                        :P_NRORGULTATU,
                        TO_DATE(:P_DTULTATU,'DD/MM/YYYY'),
                        :P_CDOPERULTATU
                    )
SQL;

            $params = [
                "P_NRHISTORICOEPI"   => $nrssthistoricoepi,
                "P_NRMOTIVODEVEPI"   => isset($row['NRMOTIVODEVEPI'])   ? $row['NRMOTIVODEVEPI'] : NULL ,
                "P_NRVINCULOM"       => isset($row['NRVINCULOM'])       ? $row['NRVINCULOM'] : NULL ,
                "P_NRORG"            => $nrorg,
                "P_NREPI"            => isset($row['NREPI'])            ? $row['NREPI'] : NULL ,
                "P_NRPARCNEGOCIO"    => isset($row['NRPARCNEGOCIO'])    ? $row['NRPARCNEGOCIO'] : NULL ,
                "P_DTENTREGAEPI"     => isset($row['DTENTREGAEPI'])     ? $row['DTENTREGAEPI'] : NULL ,
                "P_IDCTRLENTREGA"    => isset($row['IDCTRLENTREGA'])    ? $row['IDCTRLENTREGA'] : NULL ,
                "P_DTEFETIVA"        => isset($row['DTEFETIVA'])        ? $row['DTEFETIVA'] : NULL ,
                "P_QTFORNECIDAEPI"   => isset($row['QTFORNECIDAEPI'])   ? $row['QTFORNECIDAEPI'] : NULL ,
                "P_DTDEVOLUCAO"      => isset($row['DTDEVOLUCAO'])      ? $row['DTDEVOLUCAO'] : NULL ,
                "P_DSMODELOEPI"      => isset($row['DSMODELOEPI'])      ? $row['DSMODELOEPI'] : NULL ,
                "P_NRCTRLAPROVEPI"   => isset($row['NRCTRLAPROVEPI'])   ? $row['NRCTRLAPROVEPI'] : NULL ,
                "P_IDPROTECAOEFICAZ" => isset($row['IDPROTECAOEFICAZ']) ? $row['IDPROTECAOEFICAZ'] : NULL ,
                //-*--
                "P_IDATIVO"          => isset($row['IDATIVO'])          ? $row['IDATIVO'] : 'S' ,
                "P_NRORGINCLUSAO"    => isset($row['NRORGINCLUSAO'])    ? $row['NRORGINCLUSAO'] : NULL ,
                "P_DTINCLUSAO"       => isset($row['DTINCLUSAO'])       ? $row['DTINCLUSAO'] : NULL ,
                "P_CDOPERINCLUSAO"   => isset($row['CDOPERINCLUSAO'])   ? $row['CDOPERINCLUSAO'] : NULL ,
                "P_NRORGULTATU"      => isset($row['NRORGULTATU'])      ? $row['NRORGULTATU'] : NULL ,
                "P_DTULTATU"         => isset($row['DTULTATU'])         ? $row['DTULTATU'] : NULL ,
                "P_CDOPERULTATU"     => isset($row['CDOPERULTATU'])     ? $row['CDOPERULTATU'] : NULL ,
            ];

            $query = $this->entityManager->getConnection()->prepare($sql);
            $result = $query->execute($params);

            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function updateHistoricoEpi($row) {
        try {
            $nrorg = $row['NRORG'];
            $dataAtualizacao = date('d/m/Y H:i:s');

            $sql = <<<SQL
                    UPDATE SST_HISTORICOEPI
                    SET 
                        NRMOTIVODEVEPI      = :P_NRMOTIVODEVEPI,
                        NRVINCULOM          = :P_NRVINCULOM,
                        NREPI               = :P_NREPI,
                        NRPARCNEGOCIO       = :P_NRPARCNEGOCIO,
                        DTENTREGAEPI        = :P_DTENTREGAEPI,
                        IDCTRLENTREGA       = :P_IDCTRLENTREGA,
                        DTEFETIVA           = :P_DTEFETIVA,
                        QTFORNECIDAEPI      = :P_QTFORNECIDAEPI,
                        DTDEVOLUCAO         = :P_DTDEVOLUCAO,
                        DSMODELOEPI         = :P_DSMODELOEPI,
                        NRCTRLAPROVEPI      = :P_NRCTRLAPROVEPI,
                        IDPROTECAOEFICAZ    = :P_IDPROTECAOEFICAZ,
                        IDATIVO             = :P_IDATIVO,
                        NRORGINCLUSAO       = :P_NRORGINCLUSAO,
                        DTINCLUSAO          = :P_DTINCLUSAO,
                        CDOPERINCLUSAO      = :P_CDOPERINCLUSAO,
                        NRORGULTATU         = :P_NRORGULTATU,
                        DTULTATU            = :P_DTULTATU,
                        CDOPERULTATU        = :P_CDOPERULTATU
                    WHERE
                        NRORG = :P_NRORG AND NRHISTORICOEPI = :P_NRHISTORICOEPI
SQL;

            $params = [
                "P_NRHISTORICOEPI"   => $row['NRHISTORICOEPI'],
                "P_NRMOTIVODEVEPI"   => isset($row['NRMOTIVODEVEPI'])   ? $row['NRMOTIVODEVEPI']    : NULL,
                "P_NRVINCULOM"       => isset($row['NRVINCULOM'])       ? $row['NRVINCULOM']        : NULL,
                "P_NRORG"            => $nrorg,
                "P_NREPI"            => isset($row['NREPI'])            ? $row['NREPI']             : NULL,
                "P_NRPARCNEGOCIO"    => isset($row['NRPARCNEGOCIO'])    ? $row['NRPARCNEGOCIO']     : NULL,
                "P_DTENTREGAEPI"     => isset($row['DTENTREGAEPI'])     ? $row['DTENTREGAEPI']      : NULL,
                "P_IDCTRLENTREGA"    => isset($row['IDCTRLENTREGA'])    ? $row['IDCTRLENTREGA']     : NULL,
                "P_DTEFETIVA"        => isset($row['DTEFETIVA'])        ? $row['DTEFETIVA']         : NULL,
                "P_QTFORNECIDAEPI"   => isset($row['QTFORNECIDAEPI'])   ? $row['QTFORNECIDAEPI']    : NULL,
                "P_DTDEVOLUCAO"      => isset($row['DTDEVOLUCAO'])      ? $row['DTDEVOLUCAO']       : NULL,
                "P_DSMODELOEPI"      => isset($row['DSMODELOEPI'])      ? $row['DSMODELOEPI']       : NULL,
                "P_NRCTRLAPROVEPI"   => isset($row['NRCTRLAPROVEPI'])   ? $row['NRCTRLAPROVEPI']    : NULL,
                "P_IDPROTECAOEFICAZ" => isset($row['IDPROTECAOEFICAZ']) ? $row['IDPROTECAOEFICAZ']  : NULL,
                "P_IDATIVO"          => isset($row['IDATIVO'])          ? $row['IDATIVO']           : 'S',
                "P_NRORGINCLUSAO"    => isset($row['NRORGINCLUSAO'])    ? $row['NRORGINCLUSAO']     : NULL,
                "P_DTINCLUSAO"       => isset($row['DTINCLUSAO'])       ? $row['DTINCLUSAO']        : NULL,
                "P_CDOPERINCLUSAO"   => isset($row['CDOPERINCLUSAO'])   ? $row['CDOPERINCLUSAO']    : NULL,
                "P_NRORGULTATU"      => isset($row['DTINCLUSAO'])       ? $row['DTINCLUSAO']        : NULL,
                "P_DTULTATU"         => $dataAtualizacao,
                "P_CDOPERULTATU"     => isset($row['CDOPERINCLUSAO'])   ? $row['CDOPERINCLUSAO']    : NULL,
            ];

            $query = $this->entityManager->getConnection()->prepare($sql);
            $result = $query->execute($params);

            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function deleteHistoricoEpi($row) {
        try {
            $nrorg = $row['NRORG'];

            $sql = <<<SQL
                    DELETE FROM SST_HISTORICOEPI
                    WHERE NRORG = :P_NRORG AND NRHISTORICOEPI = :P_NRHISTORICOEPI
SQL;

            $params = [
                "P_NRHISTORICOEPI"   => $row['NRHISTORICOEPI'],
                "P_NRORG"            => $nrorg,
            ];

            $query = $this->entityManager->getConnection()->prepare($sql);
            $result = $query->execute($params);

            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }
}